﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Flight_Data_Recorder
{
    
    public partial class ExportKML : Form
    {
        string KMLpath = "";
        string LogPath = "";
        public ExportKML(string KPath, string DataPath)
        {
            InitializeComponent();
            KMLpath = KPath;
            LogPath = DataPath;
        }

        public void cmdExportComfirm_Click(object sender, EventArgs e)
        {
            if (checkBox1.Checked = true)
            {
                int pointFrequency = Convert.ToInt16(numericUpDown1.Value);
                CreateKML(KMLpath, LogPath, pointFrequency);
            }
            else
            {
                CreateKML(KMLpath, LogPath, 0);
            }
        }

        private void ExportKML_Load(object sender, EventArgs e)
        {
            lblPath.Text = String.Format("Saving KML file to: {0}", KMLpath);
        }

        public void UpdateProgressBar(int value, int total)
        {
            progressBar1.Maximum = total;
            progressBar1.Value = value;
        }

        public string Headers(string path)
        {

            return "<?xml version='1.0' encoding='UTF-8'?> <kml xmlns='http://www.opengis.net/kml/2.2'> <Document><name>KmlFile</name>" +
                    "<Style id='yellowLineGreenPoly'> <LineStyle><color>7f00ffff</color><width>4</width></LineStyle><PolyStyle><color>7f00ff00</color></PolyStyle></Style>"+
                    "<Placemark> <name>Flight Data</name><visibility>1</visibility><description>Flight Data Recorded by Flight Data Recorder. Data made from file " + path + 
                    "</description><styleUrl>#yellowLineGreenPoly</styleUrl><LineString><extrude>1</extrude><tessellate>1</tessellate><altitudeMode>absolute</altitudeMode>"+
                    "<coordinates>";          
        }
        public void CreateKML(string pathKML, string pathLog, int PointFrequency)
        {
            string fileText = "";
            string footers = "</coordinates></LineString></Placemark>";
            
            fileText += Headers(pathLog);
            int TotalRecordsToWrite = DataReadWrite.GetNumberOfLinesInFile(pathLog);
            for (int i = 1; i <= TotalRecordsToWrite - 1; i = i + 1)//runs for every row in the log file (one for every record)
            {
                double CurrAlt = DataReadWrite.GetDataLine(i, pathLog)[3]/3.2808399;//gets the altitude and transfers it into meters
                string LineToWrite = DataReadWrite.GetDataLine(i, pathLog)[2].ToString() + ",";//adds the longitiude
                LineToWrite += DataReadWrite.GetDataLine(i, pathLog)[1].ToString() + ",";//adds the latitude
                LineToWrite += CurrAlt.ToString();//Adds the alt in meters
                fileText += LineToWrite + "\n";

                if (PointFrequency > 0)//if we are writing the data points 
                {
                    UpdateProgressBar(i, TotalRecordsToWrite*2);//save room for them to write
                }
                else
                {
                    UpdateProgressBar(i, TotalRecordsToWrite);
                }
            }

            if (PointFrequency > 0)
            {
                fileText += footers;//writes </coordinates></LineString></Placemark>
                fileText += "<Folder><name>Position Points</name><visiblity>0</visiblity>";
                for (int i = 1; i <= TotalRecordsToWrite - 1; i = i + PointFrequency)//runs for every row in the log file (skips due to point frequency)
                {
                    double CurrAlt = DataReadWrite.GetDataLine(i, pathLog)[3] / 3.2808399;//gets the altitude and transfers it into meters
                    string CurrLat = DataReadWrite.GetDataLine(i, pathLog)[1].ToString();
                    string CurrLong = DataReadWrite.GetDataLine(i, pathLog)[2].ToString();//adds the longitiude
                    double[] CurrData = DataReadWrite.GetDataLine(i, pathLog);

                    fileText += string.Format("<Placemark><name>Data Point {0}</name><visiblity>0</visiblity>", i);
                    fileText += string.Format("<description>RECORDED AIRCRAFT DATA <br />Data ID: {0} <br />Lat: {1} <br />Long: {2} <br />Alt: {3} <br />Head: {4} <br />Bank: {5} <br />Pitch: {6} <br />Airspeed: {7} <br />VertSpeed: {8}</description>",
                        CurrData[0].ToString(), CurrData[1].ToString(), CurrData[2].ToString(), CurrData[3].ToString(), CurrData[4].ToString(), CurrData[5].ToString(), CurrData[6].ToString(), CurrData[7].ToString(), CurrData[8].ToString());//decription
                    fileText += "<Style><IconStyle><Icon><href>root://icons/palette-2.png</href></Icon></IconStyle></Style>";//Icon settings
                    fileText += string.Format("<Point><altitudeMode>absolute</altitudeMode><coordinates>{0},{1},{2}</coordinates></Point>", CurrLong, CurrLat, CurrAlt);
                    fileText += "</Placemark>";

                    UpdateProgressBar(i+TotalRecordsToWrite, TotalRecordsToWrite*2);
                } 
            }

            fileText += "</Folder>";
            fileText += "</Document>";
            fileText += "</kml>";
            StreamWriter writer = new StreamWriter(pathKML);
            writer.Write(fileText);
            writer.Close();
            this.Close();
            //<Folder>
            //    <name>Position Points</name>
            //    <visiblity>0</visiblity>
            //  <Placemark>
            //      <name>Data Point 1</name>
            //      <visiblity>0</visiblity>
            //      <description>RECORDED AIRCRAFT DATA
            //       Data ID: 1 Lat: 18.0351262967088 Long: -63.1503138712236 Alt: 479.163679197932 Head: -263.427265919745 Bank: 0.370897958055139 Pitch: -1.38840777799487 Airspeed: 163.2734375 VertSpeed: -35.051998878336</description>
            //      <Style>
            //          <IconStyle>
            //              <Icon>
            //                  <href>root://icons/palette-2.png</href>
            //              </Icon>
            //          </IconStyle>
            //      </Style>
            //      <Point>
            //          <altitudeMode>absolute</altitudeMode>
            //          <coordinates>-63.1503138712236,18.0351262967088,146.049089197535</coordinates>
            //      </Point>
            //  </Placemark>
            // </Folder>
            //</kml>
        }
    }
}
